@@ -12,7 +12,7 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # classes.py
-# Copyright (C) 2014-2020 Fracpete (pythonwekawrapper at gmail dot com)
+# Copyright (C) 2014-2021 Fracpete (pythonwekawrapper at gmail dot com)
 
 import os
 import inspect
@@ -65,10 +65,33 @@ def get_jclass(classname):
     try:
         return javabridge.class_for_name(classname)
     except:
-        return javabridge.static_call(
-            "Lweka/core/ClassHelper;", "forName",
-            "(Ljava/lang/Class;Ljava/lang/String;)Ljava/lang/Class;",
-            javabridge.class_for_name("java.lang.Object"), classname)
+        try:
+            return javabridge.static_call(
+                "Lweka/core/ClassHelper;", "forName",
+                "(Ljava/lang/Class;Ljava/lang/String;)Ljava/lang/Class;",
+                javabridge.class_for_name("java.lang.Object"), classname)
+        except:
+            return javabridge.static_call(
+                "Lweka/core/ClassHelper;", "newInstance",
+                "(Ljava/lang/String;[Ljava/lang/Class;[Ljava/lang/Object;)Ljava/lang/Object;",
+                classname, None, None)
+
+
+def get_enum(classname, enm):
+    """
+    Returns the instance of the enum.
+
+    :param classname: the classname of the enum
+    :type classname: str
+    :param enm: the name of the enum element to return
+    :type enm: str
+    :return: the enum instance
+    :rtype: JB_Object
+    """
+    return javabridge.static_call(
+        "Lweka/core/ClassHelper;", "getEnum",
+        "(Ljava/lang/String;Ljava/lang/String;)Ljava/lang/Object;",
+        classname, enm)
 
 
 def get_static_field(classname, fieldname, signature):
@@ -163,12 +186,27 @@ def list_property_names(obj):
     if isinstance(obj, JavaObject):
         obj = obj.jobject
 
-    return typeconv.string_array_to_list(
+    return typeconv.jstring_array_to_list(
         javabridge.static_call(
             "Lweka/core/ClassHelper;", "listPropertyNames",
             "(Ljava/lang/Object;)[Ljava/lang/String;",
             obj))
 
+
+def new_instance(classname):
+    """
+    Instantiates an object of the specified class.
+
+    :param classname: the name of the class to instantiate
+    :type classname: str
+    :return: the object
+    :rtype: JB_Object
+    """
+    return javabridge.static_call(
+        "Lweka/core/ClassHelper;", "newInstance",
+        "(Ljava/lang/String;[Ljava/lang/Class;[Ljava/lang/Object;)Ljava/lang/Object;",
+        classname, None, None)
+
 class Stoppable(object):
     """
     Classes that can be stopped.
@@ -653,20 +691,24 @@ class JavaObject(JSONObject):
             raise TypeError("Object does not implement or subclass " + intf_or_class + ": " + get_classname(jobject))
 
     @classmethod
-    def new_instance(cls, classname):
+    def new_instance(cls, classname, options=None):
         """
         Creates a new object from the given classname using the default constructor, None in case of error.
 
         :param classname: the classname in Java notation (eg "weka.core.DenseInstance")
         :type classname: str
+        :param options: the list of options to use, ignored if None
+        :type options: list
         :return: the Java object
         :rtype: JB_Object
         """
         try:
+            if options is None:
+                options = []
             return javabridge.static_call(
                 "Lweka/core/Utils;", "forName",
                 "(Ljava/lang/Class;Ljava/lang/String;[Ljava/lang/String;)Ljava/lang/Object;",
-                javabridge.class_for_name("java.lang.Object"), classname, [])
+                javabridge.class_for_name("java.lang.Object"), classname, options)
         except JavaException as e:
             print("Failed to instantiate " + classname + ": " + str(e))
             suggestions = suggest_package(classname, exact=True)
@@ -917,11 +959,9 @@ class Enum(JavaObject):
         :type member: str
         """
         if jobject is None:
-            enumclass = get_jclass(classname=enum)
-            jobject = javabridge.static_call(
-                "java/lang/Enum", "valueOf",
-                "(Ljava/lang/Class;Ljava/lang/String;)Ljava/lang/Enum;",
-                enumclass, member)
+            if enum is None:
+                raise Exception("Enum member cannot be None if no JB_Object instance provided!")
+            jobject = get_enum(enum, member)
         super(Enum, self).__init__(jobject)
 
     @property
@@ -1112,7 +1152,7 @@ class OptionHandler(JavaObject, Configurable):
         :rtype: list
         """
         if self.is_optionhandler:
-            return typeconv.string_array_to_list(javabridge.call(self.jobject, "getOptions", "()[Ljava/lang/String;"))
+            return typeconv.jstring_array_to_list(javabridge.call(self.jobject, "getOptions", "()[Ljava/lang/String;"))
         else:
             return []
 
@@ -1125,7 +1165,7 @@ class OptionHandler(JavaObject, Configurable):
         :type options: list
         """
         if self.is_optionhandler:
-            javabridge.call(self.jobject, "setOptions", "([Ljava/lang/String;)V", typeconv.string_list_to_array(options))
+            javabridge.call(self.jobject, "setOptions", "([Ljava/lang/String;)V", typeconv.string_list_to_jarray(options))
 
     def to_commandline(self):
         """
@@ -1636,7 +1676,7 @@ def split_options(cmdline):
     :return: the split list of commandline options
     :rtype: list
     """
-    return typeconv.string_array_to_list(
+    return typeconv.jstring_array_to_list(
         javabridge.static_call(
             "Lweka/core/Utils;", "splitOptions",
             "(Ljava/lang/String;)[Ljava/lang/String;",
@@ -1973,7 +2013,7 @@ class SetupGenerator(OptionHandler):
 
     def __init__(self, jobject=None, options=None):
         """
-        Initializes the specified classifier using its classname or the supplied JB_Object.
+        Initializes the specified setup generator.
 
         :param jobject: the JB_Object to use
         :type jobject: JB_Object
@@ -2058,6 +2098,50 @@ class SetupGenerator(OptionHandler):
         return result
 
 
+class Date(JavaObject):
+    """
+    Wraps a java.util.Date object.
+    """
+
+    def __init__(self, jobject=None, msecs=None):
+        """
+        Initializes the specified Date. Uses the current timestamp if no jobject or msecs provided.
+
+        :param jobject: the JB_Object to use
+        :type jobject: JB_Object
+        :param msecs: the milli-seconds since 1970
+        :type msecs: long
+        """
+        if jobject is None:
+            if msecs is None:
+                jobject = javabridge.make_instance("java/util/Date", "()V")
+            else:
+                jobject = javabridge.make_instance("java/util/Date", "(J)V", msecs)
+        else:
+            self.enforce_type(jobject, "java.util.Date")
+        super(Date, self).__init__(jobject=jobject)
+
+    @property
+    def time(self):
+        """
+        Returns the stored milli-seconds.
+
+        :return: the milli-seconds
+        :rtype: long
+        """
+        return javabridge.call(self.jobject, "getTime", "()J")
+
+    @time.setter
+    def time(self, msecs):
+        """
+        Sets the milli-seconds.
+
+        :param msecs: the milli-seconds to use
+        :return:
+        """
+        javabridge.call(self.jobject, "setTime", "(J)V", msecs)
+
+
 def load_suggestions():
     """
     Loads the class/package suggestions, if necessary.
